from datetime import datetime
import json
import string

import watchdog

class MessangerEventHandler(watchdog.events.FileSystemEventHandler):
    
    def __init__(self, wsMessanger, reactor, base_dir):
        self.wsMessanger = wsMessanger
        self.reactor = reactor
        self.base_dir = base_dir
        super(watchdog.events.FileSystemEventHandler, self).__init__()
    
    def on_any_event(self, event):
        print event.src_path, event.event_type
        self.reactor.callFromThread(self.wsMessanger.notify_clients,
                             (self._jsonize_event(event)))

    def _jsonize_event(self, event):
        rel_path = string.replace(event.src_path, self.base_dir, '')
        return json.dumps(
            {'event':event.event_type,
             'obj':rel_path,
             'time':str(datetime.utcnow()) + ' UTC'})



#!/usr/bin/env python

'''
Parts of this code is reused from the samples at 
http://www.tavendo.de/autobahn/tutorial/broadcast.html
and is Copyright 2011 Tavendo GmbH
'''

import logging
import sys
import os
import time

from watchdog.observers import Observer
from watchdog.events import LoggingEventHandler
from twisted.internet import reactor
from autobahn.websocket import WebSocketServerFactory, WebSocketServerProtocol, listenWS, WebSocketProtocol, ConnectionRequest

from MessangerEventHandler import MessangerEventHandler

from twisted.python import log

class FreshyServerProtocol(WebSocketServerProtocol):
    def connectionMade(self):
        print 'connection made'
        WebSocketServerProtocol.connectionMade(self)

    def onOpen(self):
        WebSocketServerProtocol.onOpen(self)
        self.factory.register(self)
        print 'Connection opened to', self.peerstr
    
    def sendFSEvent(self, json):
        WebSocketProtocol.sendMessage(self, json)
        print 'to', self.peerstr

    def onClose(self, wasClean, code, reason):
        print 'closed', self.peerstr
        WebSocketServerProtocol.onClose(self, wasClean,
                                        code, reason)
        self.factory.unregister(self)


class FreshyServerFactory(WebSocketServerFactory):
    protocol = FreshyServerProtocol

    def __init__(self, url='ws://localhost', port=4444):
        addr = url + ':' + str(port)
        print 'listening on', addr
        WebSocketServerFactory.__init__(self, addr)
        self.clients = []

    def register(self, client):
        if not client in self.clients:
            print 'registered client', client.peerstr
            self.clients.append(client)

    def unregister(self, client):
        if client in self.clients:
            print 'unregistered client', client.peerstr
            self.clients.remove(client)
        self._printConnected()

    def _printConnected(self):
        print 'connected clients:[',
        for client in self.clients:
            print client.peerstr + ',',
        print ']'

    # broadcasts a json msg to all clients
    def notify_clients(self, message):
        print 'broadcasting ', message
        for c in self.clients:
            c.sendFSEvent(message)
 
if __name__ == '__main__':
    if len(sys.argv) < 2:
        print 'Usage: python freshy-server.py <dirs>'
        sys.exit(1)

    observers = []

    log.startLogging(sys.stdout)
    ffactory = FreshyServerFactory("ws://localhost", 4444)
    ffactory.protocol = FreshyServerProtocol
    listenWS(ffactory)
    
    for arg in sys.argv[1:]:
        dir_path = os.path.abspath(arg)
        if not os.path.exists(dir_path):
            print dir_path, 'does not exist.'
            sys.exit(1)
        if not os.path.isdir(dir_path):
            print dir_path, 'is not a directory.'
            sys.exit(1)
    
        event_handler = MessangerEventHandler(ffactory, reactor, os.getcwd())
        observer = Observer()
        observer.schedule(event_handler, path=dir_path, recursive=True)
        observer.start()
        observers.append(observer)
        
    try:
        reactor.run()
    except KeyboardInterrupt:
        for obs in observers:
            obs.stop()
        reactor.stop()
        print '\nbye'
        sys.exit(1)
    




from setuptools import setup

setup(
    name = 'freshy-server',
    packages = ['freshy-server'],
    version = '0.2.3',
    description = 'Page refresh tool for web developers.',
    author = 'Lucas Chi',
    author_email = 'chi.lucas@gmail.com',
    url = 'https://github.com/lchi/freshy-server',
    classifiers = [
        'Programming Language :: Python',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Topic :: Internet :: WWW/HTTP :: Browsers',
        'Topic :: Software Development',
        ],
    requires = [
        'twisted',
        'autobahn',
        'watchdog',
        ],
    install_requires = [
        'twisted',
        'autobahn',
        'watchdog',
        ],
    scripts=['bin/freshy-server'],
    long_description = """\
Please see https://github.com/lchi/freshy-server, or the README for more information.
""",
)
